# -*- encoding: utf8 -*-

import codecs
from decimal import Decimal
from datetime import datetime
import re
from SigUtils.cnab import errors
from unicodedata import normalize
from SigUtils.formatastring import formataString


class Evento(object):
    def __init__(self, banco, codigo_evento):
        self._segmentos = []
        self.banco = banco
        self.codigo_evento = codigo_evento
        self._codigo_lote = None

    def adicionar_segmento(self, segmento):
        self._segmentos.append(segmento)
        for segmento in self._segmentos:
            segmento.servico_codigo_movimento = self.codigo_evento

    @property
    def segmentos(self):
        return self._segmentos

    def __getattribute__(self, name):
        for segmento in object.__getattribute__(self, '_segmentos'):
            if hasattr(segmento, name):
                return getattr(segmento, name)
        return object.__getattribute__(self, name)

    def __str__(self):
        return '\r\n'.join(str(seg) for seg in self._segmentos)

    def __len__(self):
        return len(self._segmentos)

    @property
    def codigo_lote(self):
        return self._codigo_lote

    @codigo_lote.setter
    def codigo_lote(self, valor):
        self._codigo_lote = valor
        for segmento in self._segmentos:
            segmento.controle_lote = valor

    def atualizar_codigo_registros(self, last_id):
        current_id = last_id
        for segmento in self._segmentos:
            current_id += 1
            segmento.servico_numero_registro = current_id
        return current_id


class Lote(object):
    def __init__(self, banco, header=None, trailer=None, tamreg=240):
        self.banco = banco
        self.header = header
        self.trailer = trailer
        self._codigo = None
        if self.tamReg != 400:
            self.trailer.quantidade_registros = 2
        self._eventos = []

    @property
    def codigo(self):
        return self._codigo

    @codigo.setter
    def codigo(self, valor):
        self._codigo = valor
        self.header.controle_lote = valor
        self.trailer.controle_lote = valor
        self.atualizar_codigo_eventos()

    def atualizar_codigo_eventos(self):
        for evento in self._eventos:
            evento.codigo_lote = self._codigo

    def atualizar_codigo_registros(self):
        last_id = 0
        for evento in self._eventos:
            last_id = evento.atualizar_codigo_registros(last_id)

    @property
    def eventos(self):
        return self._eventos

    def adicionar_evento_pag(self, evento):
        if not isinstance(evento, Evento):
            raise TypeError

        self._eventos.append(evento)
        self.trailer.quantidade_registros += len(evento)
        if self.trailer.valor_total_pago == None:
            self.trailer.valor_total_pago = Decimal(0)
        self.trailer.valor_total_pago += evento.valor_pagamento if evento.tipo_movimento in (0, 1, 2, 3) else 0
        self.atualizar_codigo_registros()

        if self._codigo:
            self.atualizar_codigo_eventos()

    def adicionar_evento_cob(self, evento):
        if not isinstance(evento, Evento):
            raise TypeError

        self._eventos.append(evento)
        if self.tamReg != 400:
            self.trailer.quantidade_registros += len(evento)
        self.atualizar_codigo_registros()

        if self._codigo:
            self.atualizar_codigo_eventos()

    def __str__(self):
        if not self._eventos:
            raise errors.NenhumEventoError()

        result = []
        if self.tamReg != 400:
            result.append(str(self.header))
        result.extend(str(evento) for evento in self._eventos)
        if self.tamReg != 400:
            result.append(str(self.trailer))
        return '\r\n'.join(result)

    def __len__(self):
        if self.banco.registros and self.tamReg == 400:
            return 1
        return self.trailer.quantidade_registros


class Arquivo(object):
    def __init__(self, banco, **kwargs):
        """Arquivo Cnab."""
        self._lotes = []
        self.banco = banco
        arquivo = kwargs.get('arquivo')
        if isinstance(arquivo, codecs.StreamReaderWriter):
            return self.carregar_retorno(arquivo)


    def setDadosEmpresa(self, cnpj, nome, endereco, numero, complemento, cidade, cep, uf):
        if isinstance(cnpj, str):
            cnpj = int(re.sub('[^0-9]', '', cnpj))
        if isinstance(numero, str):
            numero = int(re.sub('[^0-9]', '', numero))
        if isinstance(cep, str):
            cep = int(re.sub('[^0-9]', '', cep))
        try:
            nome = normalize('NFKD', nome.decode('utf-8')).encode('ASCII', 'ignore')
        except:
            nome = normalize('NFKD', nome).encode('ASCII', 'ignore')
        try:
            endereco = normalize('NFKD', endereco.decode('utf-8')).encode('ASCII', 'ignore')
        except:
            endereco = normalize('NFKD', endereco).encode('ASCII', 'ignore')
        try:
            cidade = normalize('NFKD', cidade.decode('utf-8')).encode('ASCII', 'ignore')
        except:
            cidade = normalize('NFKD', cidade).encode('ASCII', 'ignore')
        if not complemento:
            complemento = ""
        try:
            complemento = normalize('NFKD', complemento.decode('utf-8')).encode('ASCII', 'ignore')
        except:
            complemento = normalize('NFKD', complemento).encode('ASCII', 'ignore')
        self._dadosEmpresa_cnpj = cnpj
        self._dadosEmpresa_nome = nome
        self._dadosEmpresa_endereco = endereco
        self._dadosEmpresa_endereco_numero = numero
        self._dadosEmpresa_endereco_cidade = cidade
        self._dadosEmpresa_endereco_complemento = complemento
        self._dadosEmpresa_endereco_cep = cep
        self._dadosEmpresa_endereco_uf = uf
        return True

    def setDadosConta(self, conta, agencia, dv):
        if isinstance(conta, str):
            conta = int(re.sub('[^0-9]', '', conta))
        if isinstance(agencia, str):
            agencia = int(re.sub('[^0-9]', '', agencia))
        if isinstance(dv, str):
            dv = int(re.sub('[^0-9]', '', dv))
        self._dadosConta_conta = conta
        self._dadosConta_agencia = agencia
        self._dadosConta_agencia_dv = dv
        return True

    @property
    def lotes(self):
        return self._lotes


    def adicionar_lote(self, lote):
        if not isinstance(lote, Lote):
            raise TypeError('Objeto deve ser instancia de "Lote"')

        self._lotes.append(lote)
        lote.codigo = len(self._lotes)

        if self.tamReg != 400:
            lote.codigo = len(self._lotes)
            # Incrementar numero de lotes no trailer do arquivo
            self.trailer.totais_quantidade_lotes += 1

            # Incrementar numero de registros no trailer do arquivo
            self.trailer.totais_quantidade_registros += len(lote)

    def escrever(self, file_):
        file_.write(str(self).encode('ascii').decode('ascii'))

    def __str__(self):
        if not self._lotes:
            raise errors.ArquivoVazioError()

        result = []
        result.append(str(self.header))
        result.extend(str(lote) for lote in self._lotes)
        result.append(str(self.trailer))
        # Adicionar elemento vazio para arquivo terminar com \r\n
        result.append('')
        return '\r\n'.join(result)


class ArquivoCob(Arquivo):
    def __init__(self, banco, _tamReg, **kwargs):
        """Arquivo Cnab."""
        self.tamReg = _tamReg
        super(ArquivoCob, self).__init__(banco, **kwargs.get('arquivo') or {})
        self.header = eval('self.banco.registros.HeaderCobArquivo_'+str(self.tamReg))(
            **kwargs.get('arquivo') or {})
        self.headerLote = eval('self.banco.registros.HeaderCobArquivo_'+str(self.tamReg))(
            **kwargs.get('lote') or {})
        self.trailer = eval('self.banco.registros.TrailerCobArquivo_'+str(self.tamReg))(
            **kwargs.get('arquivo') or {})
        self.trailer.totais_quantidade_lotes = 0
        self.trailer.totais_quantidade_registros = 2
        self.detailT = []
        self.detailU = []

        if self.header.arquivo_data_de_geracao is None:
            now = datetime.now()
            self.header.arquivo_data_de_geracao = int(now.strftime("%d%m%Y"))

        if self.header.arquivo_hora_de_geracao is None:
            if now is None:
                now = datetime.now()
            self.header.arquivo_hora_de_geracao = int(now.strftime("%H%M%S"))

    def encontrar_lote(self, codigo_servico):
        for lote in self.lotes:
            if lote.header.servico_servico == codigo_servico:
                return lote
        return False

    def carregar_retorno(self, arquivo):

        lote_aberto = None
        evento_aberto = None
        titulos = []

        c = 0
        for linha in arquivo:
            if len(linha) < 7:
                continue
            vals_titulos = {}
            tipo_registro = linha[7]

            if tipo_registro == '0':
                self.header = eval('self.banco.registros.HeaderCobArquivo_'+str(self.tamReg))()
                self.header.carregar(linha)

            elif tipo_registro == '1':
                codigo_servico = linha[9:11]

                if codigo_servico == '01':
                    header_lote = eval('self.banco.registros.HeaderCobLote_'+str(self.tamReg))()
                    trailer_lote = eval('self.banco.registros.TrailerCobLote_'+str(self.tamReg))()
                    header_lote.carregar(linha)
                    lote_aberto = Lote(self.banco, header_lote, trailer_lote)
                    self._lotes.append(lote_aberto)

            elif tipo_registro == '3':
                tipo_segmento = linha[13]

                if tipo_segmento in ('T', 'K'):
                    if tipo_segmento == 'K':
                        tipo_segmento += 'Retorno'
                    seg = eval('self.banco.registros.Segmento%s_%s' % (tipo_segmento,
                                                                       str(self.tamReg)))()
                    seg.carregar(linha)
                    if tipo_segmento == 'T':
                        self.detailT.append(seg)

                    evento_aberto = Evento(self.banco, seg.servico_codigo_movimento)
                    lote_aberto._eventos.append(evento_aberto)
                    evento_aberto._segmentos.append(seg)
                    c += 1
                    vals_titulos = {
                        'nosso_numero': seg.nosso_numero,
                        'numero_documento': seg.numero_documento,
                        'cedente_conta': seg.cedente_conta,
                        'vencimento_titulo': formataString(seg.vencimento_titulo, 8, '0', 'E'),
                        'valor_titulo': float(seg.valor_titulo),
                        'codigo_movimento': seg.servico_codigo_movimento,
                        'carteira': seg.cobranca_carteira,
                        'valor_tarifas': float(seg.valor_tarifas)
                    }
                    try:
                        vals_titulos['id'] = int(seg.identificacao_titulo)
                    except:
                        vals_titulos['id'] = False

                elif tipo_segmento == 'U':
                    seg_u = eval('self.banco.registros.SegmentoU_'+str(self.tamReg))()
                    seg_u.carregar(linha)
                    self.detailU.append(seg_u)
                    evento_aberto._segmentos.append(seg_u)
                    evento_aberto = None

                    titulos[c - 1].update({
                        'titulo_pago': float(seg_u.titulo_pago),
                        'titulo_acrescimos': float(seg_u.titulo_acrescimos),
                        'titulo_desconto': float(seg_u.titulo_desconto),
                        'data_credito': formataString(seg_u.data_credito, 8, '0', 'E'),
                        'data_pagamento': formataString(seg_u.data_ocorrencia, 8, '0', 'E')
                    })

            elif tipo_registro == '5':
                if trailer_lote is not None:
                    lote_aberto.trailer.carregar(linha)
                else:
                    raise Exception

            elif tipo_registro == '9':
                self.trailer = eval('self.banco.registros.TrailerCobArquivo_'+str(self.tamReg))()
                self.trailer.carregar(linha)
            if vals_titulos:
                titulos.append(vals_titulos)
        return titulos

    def incluir_cobranca(self, file, kwargs):
        # 1 eh o codigo de cobranca
        codigo_evento = 1
        evento = Evento(self.banco, codigo_evento)
        vr_total_titulos = 0
        qtd_total_titulos = 0

        for arg in kwargs:
            if self.tamReg == 400:
                transacao_t1 = eval('self.banco.registros.TransacaoTipo1_'+str(
                    self.tamReg))(**arg)
                evento.adicionar_segmento(transacao_t1)
            elif arg.get('cobranca_cadastramento'):  # Identifica banco 399 - HSBC
                # seg_j = eval('self.banco.registros.SegmentoJ_'+str(self.tamReg))(**arg)
                # evento.adicionar_segmento(seg_j)
                seg_k = eval('self.banco.registros.SegmentoK_'+str(self.tamReg))(**arg)
                evento.adicionar_segmento(seg_k)
            else:
                seg_p = eval('self.banco.registros.SegmentoP_'+str(self.tamReg))(**arg)
                evento.adicionar_segmento(seg_p)

                seg_q = eval('self.banco.registros.SegmentoQ_'+str(self.tamReg))(**arg)
                evento.adicionar_segmento(seg_q)

                seg_r = eval('self.banco.registros.SegmentoR_'+str(self.tamReg))(**arg)
                if seg_r.necessario():
                    evento.adicionar_segmento(seg_r)
            vr_total_titulos += float(arg.get('valor_titulo'))
            qtd_total_titulos += 1

        lote_cobranca = self.encontrar_lote(codigo_evento)

        if not lote_cobranca:
            if self.tamReg == 400:
                header = trailer = None
            else:
                header = eval('self.banco.registros.HeaderCobLote_'+str(self.tamReg))(
                    **self.headerLote.todict())
                trailer = eval('self.banco.registros.TrailerCobLote_'+str(self.tamReg))()
            lote_cobranca = Lote(self.banco, header, trailer)
            self.adicionar_lote(lote_cobranca)
            if self.tamReg != 400:
                header.controlecob_numero = int('{0}{1:02}'.format(
                    self.header.arquivo_sequencia,
                    lote_cobranca.codigo))

                header.controlecob_data_gravacao = header.data_credito = self.header.arquivo_data_de_geracao
                if self.banco.codigoBC == 756: #sicoob
                    header.data_credito = 00000000

        if lote_cobranca:
            lote_cobranca.adicionar_evento_cob(evento)
            # Incrementar numero de registros no trailer do arquivo
            if self.tamReg != 400:
                lote_cobranca.trailer.valor_credito = Decimal(str(vr_total_titulos))
                lote_cobranca.trailer.cobrancasimples_quantidade_titulos = qtd_total_titulos
                lote_cobranca.trailer.cobrancasimples_valor_titulos = Decimal(str(vr_total_titulos))
        self.trailer.totais_quantidade_registros += len(evento)
        self.escrever(file)


class ArquivoPag(Arquivo):
    def __init__(self, banco, tamReg, **kwargs):
        """Arquivo Cnab."""
        self.tamReg = tamReg
        super(ArquivoPag, self).__init__(banco, **kwargs)
        self.header = eval('self.banco.registros.HeaderPagArquivo_'+str(self.tamReg))(**kwargs)
        self.trailer = eval('self.banco.registros.TrailerPagArquivo_'+str(self.tamReg))(**kwargs)
        self.trailer.totais_quantidade_lotes = 0
        self.trailer.totais_quantidade_registros = 2

        if self.header.arquivo_data_de_geracao is None:
            now = datetime.now()
            self.header.arquivo_data_de_geracao = int(now.strftime("%d%m%Y"))

        if self.header.arquivo_hora_de_geracao is None:
            if now is None:
                now = datetime.now()
            self.header.arquivo_hora_de_geracao = int(now.strftime("%H%M%S"))

    def setDadosEmpresa(self, cnpj, nome, endereco, numero, complemento, cidade, cep, uf):
        super(ArquivoPag, self).setDadosEmpresa(cnpj, nome, endereco, numero, complemento, cidade,
                                                cep, uf)
        self.header.inscricao_numero = self._dadosEmpresa_cnpj
        self.header.empresa = self._dadosEmpresa_nome
        self.header.inscricao_tipo = 2
        self.header.arquivo_codigo = 1  #Indica que é remessa
        return True

    def setDadosConta(self, conta, agencia, dv):
        super(ArquivoPag, self).setDadosConta(conta, agencia, dv)
        self.header.agencia = self._dadosConta_agencia
        self.header.conta = self._dadosConta_conta
        self.header.agencia_conta_dv = self._dadosConta_agencia_dv
        return True

    def encontrar_lote(self, tipo_pagamento, forma_pagamento):
        for lote in self.lotes:
            if lote.header.tipo_pagamento == tipo_pagamento and \
                            lote.header.forma_pagamento == forma_pagamento:
                return lote
        return False

    def carregar_retorno(self, arquivo):
        lote_aberto = None
        evento_aberto = None
        header = {}
        segmentos = []
        for linha in arquivo:
            if not linha:
                continue
            tipo_registro = linha[7]
            if self.tamReg == 400:
                tipo_registro = linha[0]

            if tipo_registro == '0':
                self.header = eval('self.banco.registros.HeaderPagArquivo_'+str(self.tamReg))()
                self.header.carregar(linha)
                header = self.header.todict()

            elif tipo_registro == '1':
                codigo_servico = linha[9:11]

                if self.tamReg == 400:
                    lote_aberto = Lote(self.banco, tamreg=self.tamReg)
                    self._lotes.append(lote_aberto)
                    seg = eval('self.banco.registros.RegistroRetTransacaoTipo1_%s' % str(
                        self.tamReg))()
                    seg.carregar(linha)
                    evento_aberto = Evento(self.banco, 'TransacaoTipo1')
                    lote_aberto._eventos.append(evento_aberto)
                    evento_aberto._segmentos.append(seg)
                    vals_titulos = {
                        'id': seg.numero_controle_participante,
                        'nosso_numero': seg.nosso_numero_inter,
                        'numero_documento': seg.numero_documento,
                        'cedente_conta': int(str(seg.identificacao_empresa_beneficiaria)[-10:]),
                        'vencimento_titulo': formataString(seg.data_vencimento_titulo, 8, '0', 'E'),
                        'valor_titulo': float(seg.valor_titulo),
                        'codigo_movimento': seg.identificacao_ocorrencia == 4 and 6
                            or seg.identificacao_ocorrencia,
                        'carteira': seg.numero_carteira,
                        'titulo_pago': float(seg.valor_pago),
                        'titulo_acrescimos': 0,
                        'titulo_desconto': 0,
                        'data_credito': formataString(seg.data_credito, 8, '0', 'E'),
                        'data_pagamento': formataString(seg.data_ocorrencia, 8, '0', 'E')
                    }
                elif codigo_servico == '01':
                    header_lote = eval('self.banco.registros.HeaderCobLote_'+str(self.tamReg))()
                    trailer_lote = eval('self.banco.registros.TrailerCobLote_'+str(self.tamReg))()
                    header_lote.carregar(linha)
                    lote_aberto = Lote(self.banco, header_lote, trailer_lote, tamreg=self.tamReg)
                    self._lotes.append(lote_aberto)

            elif tipo_registro == '3':
                tipo_segmento = linha[13]
                if tipo_segmento == 'J':
                    seg_J = eval('self.banco.registros.SegmentoJ_'+str(self.tamReg))()
                    seg_J.carregar(linha)
                    vals_segmento = seg_J.todict()
                    segmentos.append(vals_segmento)
                elif tipo_segmento == 'O':
                    seg_O = eval('self.banco.registros.SegmentoO_'+str(self.tamReg))()
                    seg_O.carregar(linha)
                    vals_segmento = seg_O.todict()
                    segmentos.append(vals_segmento)

            # elif tipo_registro == '3':
            #     tipo_segmento = linha[13]
            #     codigo_evento = linha[14:17]
            #     _ocorrencias = []
            #     _a = linha[230:]
            #     for i in range(5):
            #         _o = _a[i*2:i*2+2]
            #         if _o != '  ':
            #             _ocorrencias.append(_o)
            #     if tipo_segmento == 'J':
            #         seg_J = eval('self.banco.registros.SegmentoJ_'+str(self.tamReg))()
            #         seg_J.carregar(linha)
            #         evento_aberto = Evento(self.banco, int(codigo_evento))
            #         lote_aberto._eventos.append(evento_aberto)
            #         evento_aberto._segmentos.append(seg_J)
            #         vals_segmento = {
            #             'id': linha[182:202].strip(),
            #             'nosso_numero': linha[215:230],
            #             'fintit_pgm_banco_pagador': linha[:3],
            #             'fintit_pgm_data_pagamento': linha[144:152],
            #             'fintit_pgm_valor': float(linha[152:165] + '.' + linha[165:167]),
            #         }
            #     elif tipo_segmento == 'O':
            #         seg_O = eval('self.banco.registros.SegmentoO_'+str(self.tamReg)()
            #         seg_O.carregar(linha)
            #         evento_aberto._segmentos.append(seg_O)
            #         evento_aberto = None
            #         vals_segmento = {
            #             'id': linha[174:194].strip(),
            #             'nosso_numero': linha[215:230],
            #             'fintit_pgm_banco_pagador': linha[:3],
            #             'fintit_pgm_data_pagamento': linha[136:144],
            #             'fintit_pgm_valor': float(linha[144:157] + '.' + linha[157:159]),
            #         }
            #     if tipo_segmento in ('J', 'O'):
            #         if ('00' or 'BD' or 'AE' or 'BE' or 'CE' or 'IR') in _ocorrencias: # Pagamento efetuado / Pagamento agendado
            #             segmentos.append(vals_segmento)
            # elif tipo_registro == '5':
            #     if trailer_lote is not None:
            #         lote_aberto.trailer.carregar(linha)
            #     else:
            #         raise Exception
            #
            # elif tipo_registro == '9':
            #     self.trailer = eval('self.banco.registros.TrailerPagArquivo_'+str(self.tamReg))()
            #     self.trailer.carregar(linha)
        return {'header': header, 'segmentos': segmentos}

    def incluir_pagamento(self, segmento, tipo_pagamento, forma_pagamento):
        if str(segmento.__class__) == "<class 'SigUtils.cnab.registro.SegmentoJ'>":
            if segmento.tipo_movimento == 0:
                segmento.nosso_numero = ""
            if segmento.tipo_movimento in (517, 519, 999):
                segmento.codbar_banco_favorecido = 0
                segmento.codbar_moeda = 0
                segmento.codbar_dv = 0
                segmento.codbar_vencimento = 0
                segmento.codar_valor = 0
                segmento.codbar_campo_livre = 0
                segmento.favorecido = ""
                segmento.data_vencimento = 0
                segmento.valor_titulo = 0
                segmento.descontos = 0
                segmento.acrescimos = 0
                segmento.seu_numero = ""
                if segmento.operacao == '517':
                    segmento.data_pagamento = 0
                if segmento.operacao == '519':
                    segmento.valor_pagamento = 0
                if segmento.operacao == '999':
                    segmento.data_pagamento = 0
                    segmento.valor_pagamento = 0
        if str(segmento.__class__) == "<class 'SigUtils.cnab.registro.SegmentoO'>":
            if segmento.tipo_movimento == 0:
                segmento.nosso_numero = ""
                segmento.nota_fiscal = ""
            segmento.quantidade_moeda = 0.0
            if segmento.tipo_movimento in (517, 519, 999):
                segmento.codigo_barras = ""
                segmento.nome_concessionaria = ""
                segmento.data_vencimento = 0
                segmento.seu_numero = 0
                if segmento.operacao == '517':
                    segmento.data_pagamento = 0
                if segmento.operacao == '519':
                    segmento.valor_pagamento = 0
                if segmento.operacao == '999':
                    segmento.data_pagamento = 0
                    segmento.valor_pagamento = 0
        # 1 eh o codigo de cobranca
        codigo_evento = 1
        evento = Evento(self.banco, codigo_evento)
        evento.adicionar_segmento(segmento)

        lote_pagamento = self.encontrar_lote(tipo_pagamento, forma_pagamento)

        if not lote_pagamento:
            header = eval('self.banco.registros.HeaderPagLote_'+str(self.tamReg))(**self.header.todict())
            header.tipo_pagamento = tipo_pagamento
            header.forma_pagamento = forma_pagamento
            header.inscricao_tipo = 2
            header.inscricao_numero = self._dadosEmpresa_cnpj
            header.agencia = self._dadosConta_agencia
            header.empresa = self._dadosEmpresa_nome
            header.empresa_endereco = self._dadosEmpresa_endereco
            header.empresa_numero = self._dadosEmpresa_endereco_numero
            header.empresa_complemento = self._dadosEmpresa_endereco_complemento
            header.empresa_cidade = self._dadosEmpresa_endereco_cidade
            if self.banco.codigoBC == 237:
                header.empresa_cep = self._dadosEmpresa_endereco_cep.__str__()[:5]
                header.empresa_cep_complemento = self._dadosEmpresa_endereco_cep.__str__()[5:]
            else:
                header.empresa_cep = self._dadosEmpresa_endereco_cep
            header.empresa_uf = self._dadosEmpresa_endereco_uf

            trailer = eval('self.banco.registros.TrailerPagLote_'+str(self.tamReg))()
            lote_pagamento = Lote(self.banco, header, trailer, tamreg=self.tamReg)
            lote_pagamento.header.tipo_pagamento = tipo_pagamento
            lote_pagamento.header.forma_pagamento = forma_pagamento
            self.adicionar_lote(lote_pagamento)

        lote_pagamento.adicionar_evento_pag(evento)
        # Incrementar numero de registros no trailer do arquivo
        self.trailer.totais_quantidade_registros += len(evento)


class ArquivoDDA(Arquivo):
    def __init__(self, banco, tamReg, **kwargs):
        """Arquivo Cnab."""
        self.tamReg = tamReg
        super(ArquivoDDA, self).__init__(banco, **kwargs)
        self.header = eval('self.banco.registros.HeaderDDAArquivo_'+str(self.tamReg))(**kwargs)
        self.trailer = eval('self.banco.registros.TrailerDDAArquivo_'+str(self.tamReg))(**kwargs)
        self.trailer.totais_quantidade_lotes = 0
        self.trailer.totais_quantidade_registros = 2

        if self.header.arquivo_data_de_geracao is None:
            now = datetime.now()
            self.header.arquivo_data_de_geracao = int(now.strftime("%d%m%Y"))

        if self.header.arquivo_hora_de_geracao is None:
            if now is None:
                now = datetime.now()
            self.header.arquivo_hora_de_geracao = int(now.strftime("%H%M%S"))

    def setDadosEmpresa(self, cnpj, nome, endereco, numero, complemento, cidade, cep, uf):
        super(ArquivoDDA, self).setDadosEmpresa(cnpj, nome, endereco, numero, complemento, cidade,
                                                cep, uf)
        self.header.inscricao_numero = self._dadosEmpresa_cnpj
        self.header.empresa = self._dadosEmpresa_nome
        self.header.inscricao_tipo = 2
        self.header.arquivo_codigo = 2  # Indica que é retorno
        return True

    def setDadosConta(self, conta, agencia, dv):
        super(ArquivoDDA, self).setDadosConta(conta, agencia, dv)
        self.header.agencia = self._dadosConta_agencia
        self.header.conta = self._dadosConta_conta
        self.header.agencia_conta_dv = self._dadosConta_agencia_dv
        return True

    def encontrar_lote(self, tipo_pagamento, forma_pagamento):
        for lote in self.lotes:
            if lote.header.tipo_pagamento == tipo_pagamento and \
                            lote.header.forma_pagamento == forma_pagamento:
                return lote
        return False

    def carregar_retorno(self, arquivo):
        header = {}
        segmentos = {}
        cpf_cnpj = ''
        for linha in arquivo:
            if not linha:
                continue
            tipo_registro = linha[7]

            if tipo_registro == '0':
                self.header = eval('self.banco.registros.HeaderDDAArquivo_'+str(self.tamReg))()
                self.header.carregar(linha)
                header = self.header.todict()

            elif tipo_registro == '1':
                lote = eval('self.banco.registros.HeaderDDALote_'+str(self.tamReg))()
                lote.carregar(linha)
                lote = lote.todict()
                cpf_cnpj = lote.get('inscricao_numero') or ''
                segmentos.update({cpf_cnpj: {'G': [], 'H': []}})

            elif tipo_registro == '3':
                tipo_segmento = linha[13]
                if tipo_segmento == 'G':
                    seg_G = eval('self.banco.registros.SegmentoG_'+str(self.tamReg))()
                    seg_G.carregar(linha)
                    vals_segmento = seg_G.todict()
                    # segmentos.append(vals_segmento)
                    segmentos[cpf_cnpj]['G'].append(vals_segmento)
                elif tipo_segmento == 'H':
                    seg_H = eval('self.banco.registros.SegmentoH_'+str(self.tamReg))()
                    seg_H.carregar(linha)
                    vals_segmento = seg_H.todict()
                    # segmentos.append(vals_segmento)
                    segmentos[cpf_cnpj]['H'].append(vals_segmento)

        return {'header': header, 'segmentos': segmentos}
